<?php

namespace Andy\SimilarThreads\Repository;

use XF\Mvc\Entity\Repository;

class SimilarThreads extends Repository
{
	public function getSimilarThreads($currentNodeId, $currentThreadId, $threadTitle)
	{
		//########################################
		// get similar threads
		//########################################

		// get options
		$options = \XF::options();	
		
		// get options from Admin CP -> Options -> Similar threads -> Exclude days
		$excludeDays = $options->similarThreadsExcludeDays;

		// get options from Admin CP -> Options -> Similar threads -> Exclude forums
		$excludeForums = $options->similarThreadsExcludeForums;
		
		// get options from Admin CP -> Options -> Similar threads -> Miniumum common word length
		$minimumCommonWordLength = $options->similarThreadsMinimumCommonWordLength;
		
		// get options from Admin CP -> Options -> Similar threads -> Limit
		$limit = $options->similarThreadsLimit;
		
		// get options from Admin CP -> Options -> Similar threads -> Multibyte
		$multibyte = $options->similarThreadsMultibyte;
		
		// get options from Admin CP -> Options -> Similar threads -> Same forum
		$sameForum = $options->similarThreadsSameForum;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop characters
		$stopCharacters = $options->similarThreadsStopCharacters;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop words
		$stopWords = $options->similarThreadsStopWords;

		//########################################
		// define variables
		//########################################
		
		// set other variables
		$searchWords = array();
		$searchWord1 = '';
		$searchWord2 = '';
		$searchWord3 = '';
		$results1 = array();
		$results2 = array();
		$results3 = array();
		$results4 = array();
		$results5 = array();
		
		//########################################
		// return if currentNodeId is excluded
		//########################################

		// check for excluded forums
		if (in_array($currentNodeId, $excludeForums))
		{
			return;
		}

		//########################################
		// prepare search words
		//########################################
		
		// get stopCharactersArray
		$stopCharactersArray = explode(' ', $stopCharacters);			
		
		// remove characters			
		$threadTitle = str_replace($stopCharactersArray, ' ', $threadTitle);

		if (!$multibyte)
		{
			// put into array
			$threadTitleArray = explode(' ', $threadTitle);
		}
		
		if ($multibyte)
		{			
			// put into array
			$threadTitleArray = mb_split(' ', $threadTitle);
		} 
		
		// create stopWordsArray
		$stopWordsArray = explode(' ', $stopWords);								
 		
		// remove words from array
		foreach ($threadTitleArray as $var)
		{
			if (!$multibyte)
			{
				if (!in_array(strtolower($var), $stopWordsArray))
				{		
					// verify minimumCommonWordLength			
					if (strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}

			if ($multibyte)
			{
				if (!in_array(mb_strtolower($var), $stopWordsArray))
				{	
					// verify minimumCommonWordLength				
					if (mb_strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}
		}

		// get count
		$count = count($searchWords);

		// return if no searchWords
		if ($count == 0)
		{
			return;
		}			
		
		// continue we have a search word
		if ($count > 0)
		{				
			// get first search word
			$searchWord1 = $searchWords[0];
			
			if ($count > 1)
			{	
				// get second search word
				$searchWord2 = $searchWords[1];	
			}
			
			if ($count > 2)
			{	
				// get third search word
				$searchWord3 = $searchWords[2];	
			}							
		}
		
		//########################################
		// exclude by age
		//########################################	
		
		if ($excludeDays > 0)
		{
			// convert to Unix timestamp
			$gte = time() - (86400 * $excludeDays);
		}
		else
		{
			$gte = 0;
		}

		//########################################
		// database operations 
		//########################################
			
		// get db
		$db = \XF::db();			
		
		// declare variables
		$whereclause1 = '';
		$whereclause2 = '';
		$whereclause3 = '';
		$results = array();
		$results1 = array();
		$results2 = array();
		$results3 = array();
		$results4 = array();
        $results5 = array();
		$excludeResults1 = '';
		$excludeResults2 = '';
		$excludeResults3 = '';
		$excludeResults4 = '';
        $excludeResults5 = '';
		$resultsCount1 = array();
		$resultsCount2 = array();
		$resultsCount3 = array();
		$resultsCount4 = array();
        $resultsCount5 = array();
	
		//########################################
		// whereclause1
		// viewable node permissions
		//########################################
		
		if (!$sameForum)
		{			
			// get nodeIds
			$nodeRepo = $this->repository('XF:Node');
			$nodes = $nodeRepo->getNodeList();
			$nodeIds = $nodes->keys();
			
			// create whereclause of viewable nodes
			$whereclause1 = 'AND (node_id = ' . implode(' OR node_id = ', $nodeIds);
			$whereclause1 = $whereclause1 . ')';
		}

		//########################################
		// whereclause2
		// show results from same forum
		//########################################
						
		if ($sameForum)
		{
			$whereclause2 = "AND node_id = '$currentNodeId'";
		}
	
		//########################################
		// whereclause3
		// exclude forums
		//########################################
		
		if (!empty($excludeForums))
		{
			if (!empty($excludeForums))
			{
				// create whereclause of nodes to exclude
				$whereclause3 = 'AND (node_id <> ' . implode(' AND node_id <> ', $excludeForums);
				$whereclause3 = $whereclause3 . ')';
			}
		}				

		//########################################
		// search 1
		//########################################

		if ($searchWord1 != '' AND $searchWord2 != '' AND $searchWord3 != '')
		{

			$limit1 = $limit;

            $searchValue1 = '[[:<:]]' . addslashes($searchWord1) . '[[:>:]]';
            $searchValue2 = '[[:<:]]' . addslashes($searchWord2) . '[[:>:]]';
            $searchValue3 = '[[:<:]]' . addslashes($searchWord3) . '[[:>:]]'; 

            $results1 = $db->fetchAll("
            SELECT thread_id
            FROM xf_thread
            WHERE title REGEXP ?
            AND title REGEXP ?
            AND title REGEXP ?
            AND discussion_state = ?
            AND discussion_type <> ?
            AND post_date >= ?
            AND thread_id <> ?
			$whereclause1
			$whereclause2
			$whereclause3
			ORDER BY post_date DESC
			LIMIT ?
            ", array($searchValue1, $searchValue2, $searchValue3, 'visible', 'redirect', $gte, $currentThreadId, $limit1));
		}

		//########################################
		// search 2
		//########################################		
		
		if ($searchWord1 != '' AND $searchWord2 != '')
		{
			foreach ($results1 AS $k => $v)
			{
				$resultsCount1[] = $v['thread_id'];
				
				// exclude previously found thread_id's
				$excludeResults1 = 'AND thread_id <> ' . implode(' AND thread_id <> ', $resultsCount1);
			}
			
			$count = count($resultsCount1);	
			
			if ($count < $limit AND is_numeric($count))
			{
				$limit2 = $limit - $count;

                $searchValue1 = '[[:<:]]' . addslashes($searchWord1) . '[[:>:]]';
                $searchValue2 = '[[:<:]]' . addslashes($searchWord2) . '[[:>:]]';

                $results2 = $db->fetchAll("
                SELECT thread_id
                FROM xf_thread
                WHERE title REGEXP ?
                AND title REGEXP ?
                AND discussion_state = ?
                AND discussion_type <> ?
                AND post_date >= ?
                AND thread_id <> ?
                $whereclause1
                $whereclause2
                $whereclause3
                $excludeResults1
                ORDER BY post_date DESC
                LIMIT ?
                ", array($searchValue1, $searchValue2, 'visible', 'redirect', $gte, $currentThreadId, $limit2));
			}
		}		
		
		//########################################
		// search 3
		//########################################

		if ($searchWord1 != '')
		{
			foreach ($results2 AS $k => $v)
			{
				$resultsCount2[] = $v['thread_id'];
				
				// exclude previously found thread_id's
				$excludeResults2 = 'AND thread_id <> ' . implode(' AND thread_id <> ', $resultsCount2);
			}
			
			$count = count($resultsCount1) + count($resultsCount2);
			
			if ($count < $limit AND is_numeric($count))
			{
				$limit3 = $limit - $count;

                $searchValue1 = '[[:<:]]' . addslashes($searchWord1) . '[[:>:]]';

                $results3 = $db->fetchAll("
                SELECT thread_id
                FROM xf_thread
                WHERE title REGEXP ?
                AND discussion_state = ?
                AND discussion_type <> ?
                AND post_date >= ?
                AND thread_id <> ?
                $whereclause1
                $whereclause2
                $whereclause3
                $excludeResults2
                ORDER BY post_date DESC
                LIMIT ?
                ", array($searchValue1, 'visible', 'redirect', $gte, $currentThreadId, $limit3));
			}
		}

		//########################################
		// search 4
		//########################################
		
		if ($searchWord2 != '')
		{			
			foreach ($results3 AS $k => $v)
			{
				$resultsCount3[] = $v['thread_id'];
				
				// exclude previously found thread_id's
				$excludeResults3 = 'AND thread_id <> ' . implode(' AND thread_id <> ', $resultsCount3);
			}
			
			$count = count($resultsCount1) + count($resultsCount2) + count($resultsCount3);
			
			if ($count < $limit AND is_numeric($count))
			{
				$limit4 = $limit - $count;

                $searchValue2 = '[[:<:]]' . addslashes($searchWord2) . '[[:>:]]';

                $results4 = $db->fetchAll("
                SELECT thread_id
                FROM xf_thread
                WHERE title REGEXP ?
                AND discussion_state = ?
                AND discussion_type <> ?
                AND post_date >= ?
                AND thread_id <> ?
                $whereclause1
                $whereclause2
                $whereclause3
                $excludeResults3
                ORDER BY post_date DESC
                LIMIT ?
                ", array($searchValue2, 'visible', 'redirect', $gte, $currentThreadId, $limit4));
			} 
		}
		
		//########################################
		// search 5
		//########################################
		
		if ($searchWord3 != '')
		{			
			foreach ($results4 AS $k => $v)
			{
				$resultsCount4[] = $v['thread_id'];
				
				// exclude previously found thread_id's
				$excludeResults4 = 'AND thread_id <> ' . implode(' AND thread_id <> ', $resultsCount4);
			}
			
			$count = count($resultsCount1) + count($resultsCount2) + count($resultsCount3) + count($resultsCount4);
			
			if ($count < $limit AND is_numeric($count))
			{
				$limit5 = $limit - $count;

                $searchValue3 = '[[:<:]]' . addslashes($searchWord3) . '[[:>:]]';

                $results5 = $db->fetchAll("
                SELECT thread_id
                FROM xf_thread
                WHERE title REGEXP ?
                AND discussion_state = ?
                AND discussion_type <> ?
                AND post_date >= ?
                AND thread_id <> ?
                $whereclause1
                $whereclause2
                $whereclause3
                $excludeResults4
                ORDER BY post_date DESC
                LIMIT ?
                ", array($searchValue3, 'visible', 'redirect', $gte, $currentThreadId, $limit5));
			} 
		}	

		//########################################
		// prepare similarThreads
		//########################################
        
        // get results
        $results = array_merge($results1, $results2, $results3, $results4, $results5);
		
		// return if empty
		if (empty($results))
		{
			return;
		}
        
        // foreach condition
        foreach ($results AS $result)
        {
            $ids[] = $result['thread_id'];
        }

        // get similarThreads
        $finder = \XF::finder('XF:Thread')->whereIds($ids);
        $similarThreads = $finder->fetch();
        $similarThreads = $similarThreads->sortByList($ids);

		// return similarThreads
		return $similarThreads;
	}
	
	public function getSimilarThreadsEnhanced($currentNodeId, $currentThreadId, $threadTitle)
	{
		//########################################
		// get similar threads enhanced
		//########################################

		// get options
		$options = \XF::options();
		
		// get options from Admin CP -> Options -> Similar threads -> Exclude days
		$excludeDays = $options->similarThreadsExcludeDays;

		// get options from Admin CP -> Options -> Similar threads -> Exclude forums
		$excludeForums = $options->similarThreadsExcludeForums;
		
		// get options from Admin CP -> Options -> Similar threads -> Limit
		$limit = $options->similarThreadsLimit;
		
		// get options from Admin CP -> Options -> Similar threads -> Miniumum common word length
		$minimumCommonWordLength = $options->similarThreadsMinimumCommonWordLength;
		
		// get options from Admin CP -> Options -> Similar threads -> Multibyte
		$multibyte = $options->similarThreadsMultibyte;
		
		// get options from Admin CP -> Options -> Similar threads -> Same forum
		$sameForum = $options->similarThreadsSameForum;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop characters
		$stopCharacters = $options->similarThreadsStopCharacters;
		
		// get options from Admin CP -> Options -> Similar threads -> Stop words
		$stopWords = $options->similarThreadsStopWords;
		
		// get options from Admin CP -> Options -> Debug options -> xfesEnabled
		$xfesEnabled = @$options->xfesEnabled;
		
		// return error log
		if (!$xfesEnabled)
		{
			\XF::app()->error()->logError("Please deselect Enhanced search in Similar threads options page.");
			return;
		}

		//########################################
		// define variables
		//########################################
		
		// set other variables
		$searchWords = array();
		$searchWord1 = '';
		$searchWord2 = '';
		$searchWord3 = '';
		$excludeThreadIds = array();
		$continue = true;
		$threadIds = array();
		
		//########################################
		// return if currentNodeId is excluded
		//########################################

		// check for excluded forums
		if (in_array($currentNodeId, $excludeForums))
		{
			return;
		}

		//########################################
		// prepare search words
		//########################################
		
		// get stopCharactersArray
		$stopCharactersArray = explode(' ', $stopCharacters);			
		
		// remove characters			
		$threadTitle = str_replace($stopCharactersArray, ' ', $threadTitle);

		if (!$multibyte)
		{
			// put into array
			$threadTitleArray = explode(' ', $threadTitle);
		}
		
		if ($multibyte)
		{			
			// put into array
			$threadTitleArray = mb_split(' ', $threadTitle);
		} 
		
		// create stopWordsArray
		$stopWordsArray = explode(' ', $stopWords);								
 		
		// remove words from array
		foreach ($threadTitleArray as $var)
		{
			if (!$multibyte)
			{
				if (!in_array(strtolower($var), $stopWordsArray))
				{		
					// verify minimumCommonWordLength			
					if (strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}

			if ($multibyte)
			{
				if (!in_array(mb_strtolower($var), $stopWordsArray))
				{	
					// verify minimumCommonWordLength				
					if (mb_strlen($var) >= $minimumCommonWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}
		}

		// get count
		$count = count($searchWords);

		// return in no searchWords
		if ($count == 0)
		{
			return;
		}			
		
		// continue we have a search word
		if ($count > 0)
		{				
			// get first search word
			$searchWord1 = $searchWords[0];
			
			if ($count > 1)
			{	
				// get second search word
				$searchWord2 = $searchWords[1];	
			}
			
			if ($count > 2)
			{	
				// get third search word
				$searchWord3 = $searchWords[2];	
			}							
		}

		//########################################
		// exclude by age
		//########################################	
		
		if ($excludeDays > 0)
		{
			// convert to Unix timestamp
			$gte = time() - (86400 * $excludeDays);
		}
		else
		{
			$gte = 0;
		}

		//########################################
		// get eleasticsearch information
		//########################################
		
		// get configArray
		$configArray = $options->xfesConfig;

		// get host	
		$host = $configArray['host'];			

		// get port
		$port = $configArray['port'];
		
		// get https
		$https = $configArray['https'];

		// get index
		$index = $configArray['index'];
		
		// XF 2.1
		if (\XF::options()->currentVersionId >= '2010031')
		{
			if ($https)
			{
				$url = 'https://' . $host . ':' . $port . '/' . $index . '/_search';
			}
			else
			{
				$url = 'http://' . $host . ':' . $port . '/' . $index . '/_search';
			}
		}
		
		// XF 2.0
		if (\XF::options()->currentVersionId < '2010031')
		{
			$url = $host . ':' . $port . '/' . $index . '/xf/_search';
		}
		
		//########################################
		// get excludeNodeIds
		//########################################
		
		if (!empty($excludeForums))
		{
			foreach ($excludeForums as $value)
			{
				$excludeNodeIds[] = $value;
			}
		}
		
		if (empty($excludeForums))
		{
			$excludeNodeIds[] = 0;
		}

		//########################################
		// search 1
		//########################################

		if ($continue AND $searchWord1 AND $searchWord2 AND $searchWord3)
		{
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2 . ' ' . $searchWord3,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"term" => array(
												"discussion_id" => $currentThreadId
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl =  
				array(
					"from" => 0, "size" => $limit,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2 . ' ' . $searchWord3,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"term" => array(
												"discussion_id" => $currentThreadId
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {

                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ],
                        'exceptions' => false
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    \XF::logException($e, false, "Similar threads error: ");
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }
		
		//########################################
		// search 2
		//########################################

		if ($continue AND $searchWord1 AND $searchWord2)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit2 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit2,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit2,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1 . ' ' . $searchWord2,
										"operator" => "and"
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                      
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ],
                        'exceptions' => false
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    \XF::logException($e, false, "Similar threads error: ");
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }

		//########################################
		// search 3
		//########################################

		if ($continue AND $searchWord1)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit3 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit3,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit3,
					"query" => array(
						"bool" => array(						
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord1
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									] 
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}

			// get json
			$json = json_encode($dsl);
   
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ],
                        'exceptions' => false
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    \XF::logException($e, false, "Similar threads error: ");
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }			

		//########################################
		// search 4
		//########################################

		if ($continue AND $searchWord2)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit4 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit4,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord2
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit4,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord2
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                        
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
			
			// get json
			$json = json_encode($dsl);
            
            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ],
                        'exceptions' => false
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    \XF::logException($e, false, "Similar threads error: ");
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }

            // check condition
            if ($limit >= count($threadIds))
            {
                $continue = true;	
            }
            else
            {
                $continue = false;
            }
        }

		//########################################
		// search 5
		//########################################

		if ($continue AND $searchWord3)
		{
			// get excludeThreadIds
			if (empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
			}

			if (!empty($threadIds))
			{
				$excludeThreadIds[] = $currentThreadId;
				
				foreach ($threadIds as $value)
				{
					$excludeThreadIds[] = $value;
				}
			}
			
			$limit5 = $limit - count($threadIds);
			
			if ($sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit5,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord3
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"node" => $currentNodeId
											)
										),
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                      
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);
			}
				
			if (!$sameForum)
			{
				$dsl = 
				array(
					"from" => 0, "size" => $limit5,
					"query" => array(
						"bool" => array(
							"must" => array(
								"match" => array(
									"title" => array(
										"query" => $searchWord3
									)
								)
							),
							"filter" => array(
								"bool" => array(
									"must" => [
										array(
											"term" => array(
												"type" => 'thread'
											)
										),
										array(
											"range" => array(
												"date" => array(
													"gte" => $gte
												)
											)										
										)
									],
									"must_not" => [
										array(
											"terms" => array(
												"discussion_id" => $excludeThreadIds
											)
										),
										array(
											"terms" => array(
												"node" => $excludeNodeIds
											)
										),
										array(
											"term" => array(
												"hidden" => 'true'
											)
										)
									]                                       
								)
							)
						)
					),
					"sort" => array(
						"date" => array("order" => "desc" )
					)
				);			
			}
		
			// get json
			$json = json_encode($dsl);

            // check condition
            if (!empty($json))
            {
                // get client
                $client = \XF::app()->http()->client();

                try
                {
                    $response = $client->request('POST', $url, [
                        'body' => $json,
                        'headers' => [
                            'Content-Type' => 'application/json',
                            'Content-Length' => strlen($json)
                        ],
                        'exceptions' => false
                    ]);
                }
                catch (\GuzzleHttp\Exception\RequestException $e)
                {
                    \XF::logException($e, false, "Similar threads error: ");
                    $continue = false;
                }

                // check condition
                if ($continue)
                {
                    $results = $response->getBody()->getContents();
                    $data = json_decode($results);

                    if (!empty($data->_shards->total) AND $data->_shards->total > 0)
                    {
                        // get threadIds
                        foreach ($data->hits->hits as $var) 
                        {
                            $var = str_replace('thread-', '', $var->_id);
                            if (is_numeric($var))
                            {
                                $threadIds[] = $var;
                            }
                        }
                    }
                }
            }
		}

		//########################################
		// prepare similarThreads
		//########################################

		// return if empty
		if (empty($threadIds))
		{
			return;
		}

        // get similarThreads
        $finder = \XF::finder('XF:Thread')->whereIds($threadIds);
        $similarThreads = $finder->fetch();
        $similarThreads = $similarThreads->sortByList($threadIds);

		// return similarThreads
		return $similarThreads;
	}
}